from collections import Counter, OrderedDict
from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union

from django.urls import URLPattern, URLResolver

import coreapi
from rest_framework.fields import Field
from rest_framework.request import Request
from rest_framework.views import APIView

from .generators import BaseSchemaGenerator as BaseSchemaGenerator
from .inspectors import ViewInspector as ViewInspector

def common_path(paths: Iterable[str]) -> str: ...
def is_custom_action(action: str) -> bool: ...
def distribute_links(obj: Any) -> None: ...

INSERT_INTO_COLLISION_FMT: str

class LinkNode(OrderedDict[Any, Any]):
    links: List[Any] = ...
    methods_counter: Counter[Any] = ...
    def __init__(self) -> None: ...
    def get_available_key(self, preferred_key: str) -> str: ...

def insert_into(target: LinkNode, keys: Sequence[str], value: Any) -> None: ...

class SchemaGenerator(BaseSchemaGenerator):
    default_mapping: Dict[str, str] = ...
    coerce_method_names: Optional[Dict[str, str]] = ...
    def __init__(
        self,
        urlconf: Optional[str] = ...,
        title: Optional[str] = ...,
        description: Optional[str] = ...,
        version: Optional[str] = ...,
        patterns: Sequence[Union[URLPattern, URLResolver, Any]] = ...,
        url: Optional[str] = ...,
        endpoints: Optional[Sequence[Tuple[str, str, Any]]] = ...,
    ) -> None: ...
    def get_links(self, request: Optional[Request] = ...) -> Optional[LinkNode]: ...
    def get_schema(self, request: Optional[Request] = ..., public: bool = ...) -> Optional[coreapi.Document]: ...
    def get_keys(self, subpath: Any, method: Any, view: APIView) -> List[str]: ...
    def determine_path_prefix(self, paths: List[str]) -> str: ...

def field_to_schema(field: Field[Any, Any, Any, Any]) -> Any: ...

class AutoSchema(ViewInspector):
    def __init__(self, manual_fields: Optional[List[coreapi.Field]] = ...) -> None: ...
    def get_link(self, path: str, method: str, base_url: str) -> coreapi.Link: ...
    def get_path_fields(self, path: str, method: str) -> List[coreapi.Field]: ...
    def get_serializer_fields(self, path: str, method: str) -> List[coreapi.Field]: ...
    def get_pagination_fields(self, path: str, method: str) -> Dict[str, Any]: ...
    def get_filter_fields(self, path: str, method: str) -> List[Dict[str, Any]]: ...
    def get_manual_fields(self, path: str, method: str) -> List[coreapi.Field]: ...
    @staticmethod
    def update_fields(fields: List[coreapi.Fields], update_with: List[coreapi.Fields]) -> List[coreapi.Fields]: ...
    def get_encoding(self, path: str, method: str) -> str: ...

class ManualSchema(ViewInspector):
    def __init__(self, fields: List[coreapi.Fields], description: str = ..., encoding: Optional[str] = ...) -> None: ...
    def get_link(self, path: str, method: str, base_url: str) -> coreapi.Link: ...

def is_enabled() -> bool: ...



from typing import (
    Any,
    Callable,
    Dict,
    List,
    NamedTuple,
    Optional,
    Protocol,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

from django.core.paginator import Page, Paginator
from django.db.models import Model, QuerySet
from typing_extensions import TypedDict

from coreapi import Field as CoreAPIField
from mongoengine import QuerySet as MongoQuerySet
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.views import APIView

def _positive_int(integer_string: str, strict: bool = ..., cutoff: Optional[int] = ...) -> int: ...
def _divide_with_ceil(a: int, b: int) -> int: ...
def _get_displayed_page_numbers(current: int, final: int) -> List[Optional[int]]: ...
def _get_page_links(
    page_numbers: Sequence[Optional[int]], current: int, url_func: Callable[[int], str]
) -> List[PageLink]: ...
def _reverse_ordering(ordering_tuple: Sequence[str]) -> Tuple[str, ...]: ...

class Cursor(NamedTuple):
    offset: int
    reverse: bool
    position: Optional[int]

class PageLink(NamedTuple):
    url: Optional[str]
    number: Optional[int]
    is_active: bool
    is_break: bool

class HtmlContext(TypedDict):
    previous_url: str
    next_url: str

class HtmlContextWithPageLinks(HtmlContext):
    page_links: List[PageLink]

PAGE_BREAK: PageLink = ...

_MT = TypeVar("_MT", bound=Model)

class BasePagination:
    display_page_controls: bool = ...
    def get_paginated_response_schema(self, schema: Any) -> Any: ...
    def get_paginated_response(self, data: Any) -> Response: ...
    def get_results(self, data: Dict[str, Any]) -> Any: ...
    def get_schema_fields(self, view: APIView) -> List[Any]: ...
    def get_schema_operation_parameters(self, view: APIView) -> List[Any]: ...
    def paginate_queryset(self, queryset: QuerySet[Any], request: Request, view: Optional[APIView] = ...) -> Any: ...
    def to_html(self) -> str: ...

class PageNumberPagination(BasePagination):
    display_page_controls: bool = ...
    django_paginator_class: Type[Paginator] = ...
    invalid_page_message: str = ...
    last_page_strings: Sequence[str] = ...
    max_page_size: Optional[int] = ...
    page_query_description: str = ...
    page_query_param: str = ...
    page_size_query_description: str = ...
    page_size_query_param: Optional[str] = ...
    page_size: Optional[int] = ...
    page: Optional[Page] = ...
    request: Optional[Request] = ...
    template: str = ...
    def paginate_queryset(
        self, queryset: Union[QuerySet[Any], MongoQuerySet[Any]], request: Request, view: Optional[APIView] = ...
    ) -> Optional[List[Page]]: ...
    def get_paginated_response_schema(self, schema: Dict[str, Any]) -> Dict[str, Any]: ...
    def get_schema_fields(self, view: APIView) -> List[CoreAPIField]: ...
    def get_schema_operation_parameters(self, view: APIView) -> List[Dict[str, Any]]: ...
    def get_page_size(self, request: Request) -> Optional[int]: ...
    def get_next_link(self) -> Optional[str]: ...
    def get_previous_link(self) -> Optional[str]: ...
    def get_html_context(self) -> HtmlContextWithPageLinks: ...

class CountProtocol(Protocol):
    def count(self) -> int: ...

_T = TypeVar("_T")
_D = TypeVar("_D", bound=Model)

class LimitOffsetPagination(BasePagination):
    count: Optional[int] = ...
    default_limit: Optional[int] = ...
    limit_query_description: str = ...
    limit_query_param: str = ...
    limit: Optional[int] = ...
    max_limit: Optional[int] = ...
    offset_query_description: str = ...
    offset_query_param: str = ...
    offset: Optional[int] = ...
    request: Optional[Request] = ...
    template: str = ...
    @overload
    def paginate_queryset(
        self, queryset: QuerySet[_D], request: Request, view: Optional[APIView] = ...
    ) -> Optional[List[_D]]: ...
    @overload
    def paginate_queryset(
        self, queryset: MongoQuerySet[_T], request: Request, view: Optional[APIView] = ...
    ) -> Optional[List[_T]]: ...
    def get_limit(self, request: Request) -> Optional[int]: ...
    def get_offset(self, request: Request) -> int: ...
    def get_next_link(self) -> Optional[str]: ...
    def get_previous_link(self) -> Optional[str]: ...
    def get_html_context(self) -> HtmlContextWithPageLinks: ...
    def get_count(self, queryset: CountProtocol) -> int: ...

class CursorPagination(BasePagination):
    base_url: Optional[str] = ...
    cursor_query_description: str = ...
    cursor_query_param: str = ...
    cursor: Optional[Cursor] = ...
    has_next: Optional[bool] = ...
    has_previous: Optional[bool] = ...
    invalid_cursor_message: str = ...
    max_page_size: Optional[int] = ...
    next_position: Optional[str] = ...
    offset_cutoff: int = ...
    ordering: Union[str, List[str], Tuple[str, ...]] = ...
    page_size_query_description: str = ...
    page_size_query_param: Optional[str] = ...
    page_size: Optional[int] = ...
    page: Optional[List[Any]] = ...
    previous_position: Optional[str] = ...
    template: str = ...
    @overload
    def paginate_queryset(
        self, queryset: QuerySet[_D], request: Request, view: Optional[APIView] = ...
    ) -> Optional[List[_D]]: ...
    @overload
    def paginate_queryset(
        self, queryset: MongoQuerySet[_T], request: Request, view: Optional[APIView] = ...
    ) -> Optional[List[_T]]: ...
    def get_page_size(self, request: Request) -> Optional[int]: ...
    def get_next_link(self) -> Optional[str]: ...
    def get_previous_link(self) -> Optional[str]: ...
    def get_html_context(self) -> HtmlContext: ...
    def get_ordering(self, request: Request, queryset: QuerySet[Any], view: APIView) -> Tuple[str, ...]: ...
    def decode_cursor(self, request: Request) -> Optional[Cursor]: ...
    def encode_cursor(self, cursor: Cursor) -> str: ...
    def _get_position_from_instance(self, instance: Any, ordering: Sequence[str]) -> str: ...



from typing import Any

from coreapi.codecs.base import BaseCodec
from openapi_codec.encode import generate_swagger_object as generate_swagger_object

class OpenAPICodec(BaseCodec):
    media_type: str = ...
    format: str = ...
    def decode(self, bytes: Any, **options: Any) -> Any: ...
    def encode(self, document: Any, **options: Any) -> Any: ...



from typing import Any, List, Optional, Union

from django import template
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser

from rest_framework.request import Request

register: Any
class_re: Any

def highlight_code(parser: Any, token: Any) -> Any: ...

class CodeNode(template.Node):
    style: str = ...
    lang: Any = ...
    nodelist: Any = ...
    def __init__(self, lang: Any, code: Any) -> None: ...
    def render(self, context: Any) -> Any: ...

def with_location(fields: Any, location: Any) -> Any: ...
def form_for_link(link: Any) -> Any: ...
def render_markdown(markdown_text: Any) -> Any: ...
def get_pagination_html(pager: Any) -> Any: ...
def render_form(serializer: Any, template_pack: Optional[Any] = ...) -> Any: ...
def render_field(field: Any, style: Any) -> Any: ...
def optional_login(request: Request) -> Any: ...
def optional_docs_login(request: Request) -> Any: ...
def optional_logout(request: Request, user: Union[AnonymousUser, AbstractBaseUser]) -> Any: ...
def add_query_param(request: Request, key: str, val: Any) -> Any: ...
def as_string(value: Any) -> str: ...
def as_list_of_strings(value: Any) -> List[str]: ...
def add_class(value: Any, css_class: Any) -> Any: ...
def format_value(value: Any) -> Any: ...
def items(value: Any) -> Any: ...
def data(value: Any) -> Any: ...
def schema_links(section: Any, sec_key: Optional[Any] = ...) -> Any: ...
def add_nested_class(value: Any) -> Any: ...

TRAILING_PUNCTUATION: Any
WRAPPING_PUNCTUATION: Any
word_split_re: Any
simple_url_re: Any
simple_url_2_re: Any
simple_email_re: Any

def smart_urlquote_wrapper(matched_url: Any) -> Any: ...
def urlize_quoted_links(
    text: Any, trim_url_limit: Optional[Any] = ..., nofollow: bool = ..., autoescape: bool = ...
) -> Any: ...
def break_long_headers(header: Any) -> Any: ...



from contextlib import contextmanager
from types import TracebackType
from typing import Any, BinaryIO, ContextManager, Dict, Iterator, List, Mapping, Optional, Sequence, Tuple, Type, Union

from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.http import HttpRequest, QueryDict

from django.contrib.sessions.backends.base import SessionBase
from django.contrib.sites.models import Site
from django.urls import ResolverMatch
from rest_framework.authentication import BaseAuthentication
from rest_framework.authtoken.models import Token

from rest_framework.negotiation import BaseContentNegotiation
from rest_framework.parsers import BaseParser
from rest_framework.versioning import BaseVersioning
from rest_framework.views import APIView
from django.utils.datastructures import MultiValueDict
from django.core.files import uploadedfile, uploadhandler
from django.utils.datastructures import ImmutableList, MultiValueDict
from django.http.request import HttpHeaders

_UploadHandlerList = Union[List[uploadhandler.FileUploadHandler], ImmutableList[uploadhandler.FileUploadHandler]]

def is_form_media_type(media_type: str) -> bool: ...

class override_method(ContextManager["Request"]):
    def __init__(self, view: APIView, request: Request, method: str): ...
    def __enter__(self) -> Request: ...
    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]: ...

class WrappedAttributeError(Exception): ...

@contextmanager
def wrap_attributeerrors() -> Iterator[None]: ...

class Empty: ...

def clone_request(request: Request, method: str) -> Request: ...

class ForcedAuthentication:
    force_user: Optional[Union[AnonymousUser, AbstractBaseUser]] = ...
    force_token: Optional[str] = ...
    def __init__(
        self, force_user: Optional[Union[AnonymousUser, AbstractBaseUser]], force_token: Optional[str]
    ) -> None: ...
    def authenticate(
        self, request: Request
    ) -> Tuple[Optional[Union[AnonymousUser, AbstractBaseUser]], Optional[Any]]: ...

class Request(HttpRequest):
    parsers: Optional[Sequence[BaseParser]] = ...
    authenticators: Optional[Sequence[Union[BaseAuthentication, ForcedAuthentication]]] = ...
    negotiator: Optional[BaseContentNegotiation] = ...
    parser_context: Optional[Dict[str, Any]] = ...
    version: Optional[str]
    versioning_scheme: Optional[BaseVersioning]
    _request: HttpRequest
    GET: QueryDict = ...
    POST: QueryDict = ...
    COOKIES: Dict[str, str] = ...
    META: Dict[str, Any] = ...
    FILES: MultiValueDict[str, uploadedfile.UploadedFile] = ...
    path: str = ...
    path_info: str = ...
    method: Optional[str] = ...
    resolver_match: ResolverMatch = ...
    content_type: Optional[str] = ...
    content_params: Optional[Dict[str, str]] = ...
    user: Union[AbstractBaseUser, AnonymousUser]
    site: Site
    session: SessionBase
    encoding: Optional[str] = ...
    upload_handlers: _UploadHandlerList = ...
    def __init__(
        self,
        request: HttpRequest,
        parsers: Optional[Sequence[BaseParser]] = ...,
        authenticators: Optional[Sequence[BaseAuthentication]] = ...,
        negotiator: Optional[BaseContentNegotiation] = ...,
        parser_context: Optional[Dict[str, Any]] = ...,
    ) -> None: ...
    def get_host(self) -> str: ...
    def get_port(self) -> str: ...
    def get_full_path(self, force_append_slash: bool = ...) -> str: ...
    def get_full_path_info(self, force_append_slash: bool = ...) -> str: ...
    def get_signed_cookie(
        self, key: str, default: Any = ..., salt: str = ..., max_age: Optional[int] = ...
    ) -> Optional[str]: ...
    def get_raw_uri(self) -> str: ...
    def build_absolute_uri(self, location: Optional[str] = ...) -> str: ...
    @property
    def scheme(self) -> Optional[str]: ...
    def is_secure(self) -> bool: ...
    def is_ajax(self) -> bool: ...
    def parse_file_upload(
        self, META: Mapping[str, Any], post_data: BinaryIO
    ) -> Tuple[QueryDict, MultiValueDict[str, uploadedfile.UploadedFile]]: ...
    @property
    def headers(self) -> HttpHeaders: ...
    @property
    def body(self) -> bytes: ...
    def _load_post_and_files(self) -> None: ...
    def content_type(self) -> str: ...  # type: ignore[override]
    @property
    def stream(self) -> Any: ...
    @property
    def query_params(self) -> QueryDict: ...
    @property
    def data(self) -> Dict[str, Any]: ...
    @property  # type: ignore[override]
    def user(self) -> Union[AbstractBaseUser, AnonymousUser]: ...  # type: ignore[override]
    @user.setter
    def user(self, value: Union[AbstractBaseUser, AnonymousUser]) -> None: ...
    @property
    def auth(self) -> Union[Token, Any]: ...
    @auth.setter
    def auth(self, value: Union[Token, Any]) -> None: ...
    @property
    def successful_authenticator(self) -> Optional[Union[BaseAuthentication, ForcedAuthentication]]: ...
    @property
    def DATA(self) -> None: ...
    @property
    def POST(self) -> QueryDict: ...  # type: ignore[override]
    @property
    def FILES(self) -> Any: ...  # type: ignore [override]
    @property
    def QUERY_PARAMS(self) -> None: ...
    def force_plaintext_errors(self, value: Any) -> None: ...



from typing import NamedTuple

from coreschema.schemas import Schema

class Fields: ...

class Field(NamedTuple):
    name: str
    required: bool
    location: str
    schema: Schema

class Document: ...
class Link: ...
class Client: ...



from typing import Any, Callable, Dict, List, Optional, Tuple, overload

from django.core.cache.backends.base import BaseCache

from rest_framework.request import Request
from rest_framework.views import APIView

class BaseThrottle:
    def allow_request(self, request: Any, view: APIView) -> bool: ...
    def get_ident(self, request: Any) -> str: ...
    def wait(self) -> Optional[float]: ...

class SimpleRateThrottle(BaseThrottle):
    cache: BaseCache = ...
    cache_format: str = ...
    history: List[Any]
    key: Optional[str] = ...
    now: float
    rate: Optional[str]
    scope: Optional[str] = ...
    THROTTLE_RATES: Dict[str, Optional[str]] = ...
    timer: Callable[..., float] = ...
    def __init__(self) -> None: ...
    def get_cache_key(self, request: Request, view: APIView) -> Optional[str]: ...
    def get_rate(self) -> Optional[str]: ...
    @overload
    def parse_rate(self, rate: str) -> Tuple[int, int]: ...
    @overload
    def parse_rate(self, rate: None) -> Tuple[None, None]: ...
    def throttle_failure(self) -> bool: ...
    def throttle_success(self) -> bool: ...

class AnonRateThrottle(SimpleRateThrottle): ...
class UserRateThrottle(SimpleRateThrottle): ...

class ScopedRateThrottle(SimpleRateThrottle):
    scope_attr: str = ...



from typing import Any

email_pattern: Any
uri_pattern: Any

def validate_format(value: Any, format: Any) -> Any: ...
def unknown_format(value: Any) -> Any: ...
def validate_email(value: Any) -> Any: ...
def validate_uri(value: Any) -> Any: ...



from typing import Any, Dict, List, Optional, Sequence, Union

from django.http import HttpRequest, JsonResponse

from rest_framework.renderers import BaseRenderer
from rest_framework.request import Request

def _get_error_details(data: Any, default_code: Optional[str] = ...) -> Any: ...
def _get_codes(detail: Any) -> Any: ...
def _get_full_details(detail: Any) -> Any: ...

class ErrorDetail(str):
    code: Optional[str] = ...
    def __new__(cls, string: str, code: Optional[str] = ...) -> Any: ...

_Detail = Union[str, List[Any], Dict[str, Any]]

class APIException(Exception):
    status_code: int = ...
    default_detail: str = ...
    default_code: str = ...

    detail: _Detail
    def __init__(self, detail: Optional[_Detail] = ..., code: Optional[str] = ...) -> None: ...
    def get_codes(self) -> Any: ...
    def get_full_details(self) -> Any: ...

class ValidationError(APIException): ...
class ParseError(APIException): ...
class AuthenticationFailed(APIException): ...
class NotAuthenticated(APIException): ...
class PermissionDenied(APIException): ...
class NotFound(APIException): ...

class MethodNotAllowed(APIException):
    def __init__(self, method: str, detail: Optional[_Detail] = ..., code: Optional[str] = ...) -> None: ...

class NotAcceptable(APIException):
    available_renderers: Optional[Sequence[BaseRenderer]]
    def __init__(
        self,
        detail: Optional[_Detail] = ...,
        code: Optional[str] = ...,
        available_renderers: Optional[Sequence[BaseRenderer]] = ...,
    ) -> None: ...

class UnsupportedMediaType(APIException):
    def __init__(self, media_type: str, detail: Optional[_Detail] = ..., code: Optional[str] = ...) -> None: ...

class Throttled(APIException):
    extra_detail_singular: str = ...
    extra_detail_plural: str = ...
    def __init__(self, wait: Optional[float] = ..., detail: Optional[_Detail] = ..., code: Optional[str] = ...): ...

def server_error(request: Union[HttpRequest, Request], *args: Any, **kwargs: Any) -> JsonResponse: ...
def bad_request(
    request: Union[HttpRequest, Request], exception: Exception, *args: Any, **kwargs: Any
) -> JsonResponse: ...



from typing import Any, Dict, List, Optional, Sequence, Type, TypeVar, Union, overload

from django.db.models import Manager, Model
from django.db.models.query import QuerySet
from django.http.response import HttpResponse

from mongoengine import QuerySet as MongoQuerySet
from rest_framework import mixins, views
from rest_framework.filters import BaseFilterBackend
from rest_framework.pagination import BasePagination
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer

_MT_co = TypeVar("_MT_co", bound=Model, covariant=True)

def get_object_or_404(
    queryset: Union[Type[_MT_co], Manager[_MT_co], QuerySet[_MT_co]], *filter_args: Any, **filter_kwargs: Any
) -> _MT_co: ...

_T = TypeVar("_T")
_D = TypeVar("_D", bound=Model)

_Q = TypeVar("_Q", bound=Union[QuerySet[Any], MongoQuerySet[Any]])

class GenericAPIView(views.APIView):
    serializer_class: Optional[Type[BaseSerializer]] = ...
    lookup_field: str = ...
    lookup_url_kwarg: Optional[str] = ...
    filter_backends: Sequence[Type[BaseFilterBackend]] = ...
    pagination_class: Optional[Type[BasePagination]] = ...
    def get_object(self) -> Any: ...
    def get_serializer(self, *args: Any, **kwargs: Any) -> BaseSerializer: ...
    def get_serializer_class(self) -> Type[BaseSerializer]: ...
    def get_serializer_context(self) -> Dict[str, Any]: ...
    def filter_queryset(self, queryset: _Q) -> _Q: ...
    @property
    def paginator(self) -> Optional[BasePagination]: ...
    @overload
    def paginate_queryset(self, queryset: QuerySet[_D]) -> Optional[List[_D]]: ...
    @overload
    def paginate_queryset(self, queryset: MongoQuerySet[_T]) -> Optional[List[_T]]: ...
    def get_paginated_response(self, data: Any) -> Response: ...

class CreateAPIView(mixins.CreateModelMixin, GenericAPIView):
    def post(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class ListAPIView(mixins.ListModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class RetrieveAPIView(mixins.RetrieveModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class DestroyAPIView(mixins.DestroyModelMixin, GenericAPIView):
    def delete(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class UpdateAPIView(mixins.UpdateModelMixin, GenericAPIView):
    def put(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def patch(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class ListCreateAPIView(mixins.ListModelMixin, mixins.CreateModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class RetrieveUpdateAPIView(mixins.RetrieveModelMixin, mixins.UpdateModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def put(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def patch(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class RetrieveDestroyAPIView(mixins.RetrieveModelMixin, mixins.DestroyModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def delete(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...

class RetrieveUpdateDestroyAPIView(
    mixins.RetrieveModelMixin, mixins.UpdateModelMixin, mixins.DestroyModelMixin, GenericAPIView
):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def put(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def patch(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def delete(self, request: Request, *args: Any, **kwargs: Any) -> HttpResponse: ...



from typing import Any, Optional

def dereference(lookup_string: Any, struct: Any) -> Any: ...
def is_json_pointer(value: Any) -> Any: ...
def get_dicts(item: Any, dereference_using: Optional[Any] = ...) -> Any: ...
def get_strings(item: Any) -> Any: ...



from typing import Any, Callable, Mapping, Optional, Sequence

from django.http import HttpRequest

def preserve_builtin_query_params(url: str, request: Optional[HttpRequest] = ...) -> str: ...
def reverse(
    viewname: str,
    args: Optional[Sequence[Any]] = ...,
    kwargs: Optional[Mapping[str, Any]] = ...,
    request: Optional[HttpRequest] = ...,
    format: Optional[str] = ...,
    **extra: Any
) -> str: ...
def _reverse(
    viewname: str,
    args: Optional[Sequence[Any]] = ...,
    kwargs: Optional[Mapping[str, Any]] = ...,
    request: Optional[HttpRequest] = ...,
    format: Optional[str] = ...,
    **extra: Any
) -> str: ...

reverse_lazy: Callable[..., str] = ...



from typing import Any

from coreschema.compat import text_types as text_types

jsonschema: Any
KEYWORD_TO_TYPE: Any
TYPE_NAMES: Any
CLS_MAP: Any

def camelcase_to_snakecase(name: Any) -> Any: ...
def get_typed_schemas(data: Any) -> Any: ...
def get_composite_schemas(data: Any) -> Any: ...
def load_jsonschema(data: Any) -> Any: ...



from typing import Any, Optional

from . import renderers as renderers

def get_swagger_view(
    title: Optional[Any] = ..., url: Optional[Any] = ..., patterns: Optional[Any] = ..., urlconf: Optional[Any] = ...
) -> Any: ...



from typing import Any, Optional

def is_html_input(dictionary: Any) -> Any: ...
def parse_html_list(dictionary: Any, prefix: str = ..., default: Optional[Any] = ...) -> Any: ...
def parse_html_dict(dictionary: Any, prefix: str = ...) -> Any: ...



from typing import IO, Any, Dict, Generic, Mapping, Optional, Type, TypeVar

from django.core.files.uploadedfile import UploadedFile
from django.http import QueryDict
from django.utils.datastructures import MultiValueDict

from rest_framework.renderers import JSONRenderer

_Data = TypeVar("_Data")
_Files = TypeVar("_Files")

class DataAndFiles(Generic[_Data, _Files]):
    data: _Data
    files: _Files
    def __init__(self, data: _Data, files: _Files) -> None: ...

class BaseParser:
    media_type: str = ...
    def parse(
        self, stream: IO[Any], media_type: Optional[str] = ..., parser_context: Optional[Mapping[str, Any]] = ...
    ) -> Any: ...

class JSONParser(BaseParser):
    renderer_class: Type[JSONRenderer] = ...
    strict: bool = ...
    def parse(
        self, stream: IO[Any], media_type: Optional[str] = ..., parser_context: Optional[Mapping[str, Any]] = ...
    ) -> Dict[str, Any]: ...

class FormParser(BaseParser):
    def parse(
        self, stream: IO[Any], media_type: Optional[str] = ..., parser_context: Optional[Mapping[str, Any]] = ...
    ) -> QueryDict: ...

class MultiPartParser(BaseParser):
    def parse(
        self, stream: IO[Any], media_type: Optional[str] = ..., parser_context: Optional[Mapping[str, Any]] = ...
    ) -> DataAndFiles[QueryDict, MultiValueDict[Any, Any]]: ...

class FileUploadParser(BaseParser):
    errors: Dict[str, str] = ...
    def parse(
        self, stream: IO[Any], media_type: Optional[str] = ..., parser_context: Optional[Mapping[str, Any]] = ...
    ) -> DataAndFiles[None, Mapping[str, UploadedFile]]: ...
    def get_filename(self, stream: IO[Any], media_type: Optional[str], parser_context: Mapping[str, Any]) -> str: ...
    def get_encoded_filename(self, filename_parm: Mapping[str, Any]) -> str: ...



from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, Type, TypeVar, Union

from django.http.response import HttpResponseBase
from typing_extensions import Literal, Protocol

from rest_framework.authentication import BaseAuthentication
from rest_framework.filters import BaseFilterBackend
from rest_framework.parsers import BaseParser
from rest_framework.permissions import BasePermission
from rest_framework.renderers import BaseRenderer
from rest_framework.schemas.inspectors import ViewInspector
from rest_framework.serializers import BaseSerializer
from rest_framework.throttling import BaseThrottle

class MethodMapper(Dict[str, Any]):
    def __init__(self, action: Callable[..., Any], methods: Sequence[str]) -> None: ...
    def _map(self, method: str, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def get(self, func: Callable[..., Any]) -> Callable[..., Any]: ...  # type: ignore
    def post(self, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def put(self, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def patch(self, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def delete(self, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def head(self, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def options(self, func: Callable[..., Any]) -> Callable[..., Any]: ...
    def trace(self, func: Callable[..., Any]) -> Callable[..., Any]: ...

_LOWER_CASE_HTTP_VERBS = List[
    Union[
        Literal["get"],
        Literal["post"],
        Literal["delete"],
        Literal["put"],
        Literal["patch"],
        Literal["trace"],
        Literal["options"],
    ]
]

_MIXED_CASE_HTTP_VERBS = Sequence[
    Union[
        Literal["GET"],
        Literal["POST"],
        Literal["DELETE"],
        Literal["PUT"],
        Literal["PATCH"],
        Literal["TRACE"],
        Literal["OPTIONS"],
        Literal["get"],
        Literal["post"],
        Literal["delete"],
        Literal["put"],
        Literal["patch"],
        Literal["trace"],
        Literal["options"],
    ]
]

class ViewSetAction(Protocol[_F]):
    detail: bool
    methods: _LOWER_CASE_HTTP_VERBS
    url_path: str
    url_name: str
    kwargs: Mapping[str, Any]
    mapping: MethodMapper
    __call__: _F

_CallableViewHandler = Callable[..., HttpResponseBase]

_F = TypeVar("_F", bound=_CallableViewHandler)

# TODO(sbdchd): update these to return a Protocol with the property that gets
# attached along with __call__ set to the func

def api_view(http_method_names: Optional[_MIXED_CASE_HTTP_VERBS] = ...) -> Callable[[_F], _F]: ...

_RenderClassesParam = Sequence[Type[BaseRenderer]]

def renderer_classes(renderer_classes: _RenderClassesParam) -> Callable[[_F], _F]: ...

_ParserClassesParam = Sequence[Type[BaseParser]]

def parser_classes(parser_classes: _ParserClassesParam) -> Callable[[_F], _F]: ...

_AuthClassesParam = Sequence[Type[BaseAuthentication]]

def authentication_classes(authentication_classes: _AuthClassesParam) -> Callable[[_F], _F]: ...

_ThrottleClassesParam = Sequence[Type[BaseThrottle]]

def throttle_classes(throttle_classes: _ThrottleClassesParam) -> Callable[[_F], _F]: ...

_PermClassesParam = Sequence[Type[BasePermission]]

def permission_classes(permission_classes: _PermClassesParam) -> Callable[[_F], _F]: ...

_SchemaClassesParam = Optional[Type[ViewInspector]]

def schema(view_inspector: _SchemaClassesParam) -> Callable[[_F], _F]: ...
def action(
    methods: Optional[_MIXED_CASE_HTTP_VERBS] = ...,
    detail: bool = ...,
    url_path: Optional[str] = ...,
    url_name: Optional[str] = ...,
    suffix: Optional[str] = ...,
    name: Optional[str] = ...,
    # **kwargs expanded, might have missed a few
    serializer_class: Type[BaseSerializer] = ...,
    permission_classes: _PermClassesParam = ...,
    throttle_classes: _ThrottleClassesParam = ...,
    schema: _SchemaClassesParam = ...,
    authentication_classes: _AuthClassesParam = ...,
    renderer_classes: _RenderClassesParam = ...,
    parser_classes: _ParserClassesParam = ...,
    filter_backends: Sequence[Type[BaseFilterBackend]] = ...,
    lookup_field: str = ...,
    lookup_url_kwarg: Optional[str] = ...,
) -> Callable[[_F], _F]: ...



from typing import Any

from django.db.models import Model

from rest_framework.fields import Field
from rest_framework.views import APIView

def is_list_view(path: str, method: str, view: APIView) -> bool: ...
def get_pk_description(model: Model, model_field: Field[Any, Any, Any, Any]) -> str: ...



from typing import Any, List, NamedTuple, Optional, Pattern

from coreschema.compat import numeric_types as numeric_types
from coreschema.compat import text_types as text_types
from coreschema.formats import validate_format as validate_format
from coreschema.utils import uniq as uniq

class Error(NamedTuple):
    text: str
    index: List[str]  # type: ignore [assignment]

def push_index(errors: List[Error], key: Any) -> List[str]: ...

class Schema:
    errors: List[Error] = ...
    title: Any = ...
    description: Any = ...
    default: Any = ...
    def __init__(self, title: str = ..., description: str = ..., default: Optional[Any] = ...) -> None: ...
    def make_error(self, code: str) -> Error: ...
    def __or__(self, other: Schema) -> Union: ...
    def __and__(self, other: Schema) -> Intersection: ...
    def __xor__(self, other: Schema) -> ExclusiveUnion: ...
    def __invert__(self) -> Not: ...
    def __eq__(self, other: Schema) -> bool: ...  # type: ignore [override]

class Object(Schema):
    errors: List[Error] = ...
    additional_properties_schema: Any = ...
    properties: Any = ...
    required: Optional[bool] = ...
    max_properties: int = ...
    min_properties: int = ...
    pattern_properties: Any = ...
    additional_properties: Any = ...
    pattern_properties_regex: Any = ...
    def __init__(
        self,
        properties: Optional[Any] = ...,
        required: Optional[bool] = ...,
        max_properties: Optional[Any] = ...,
        min_properties: Optional[Any] = ...,
        pattern_properties: Optional[Any] = ...,
        additional_properties: bool = ...,
        **kwargs: Any
    ) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Array(Schema):
    errors: List[Error] = ...
    items: Schema = ...
    max_items: int = ...
    min_items: int = ...
    unique_items: int = ...
    additional_items: int = ...
    def __init__(
        self,
        items: Optional[Any] = ...,
        max_items: Optional[Any] = ...,
        min_items: Optional[Any] = ...,
        unique_items: bool = ...,
        additional_items: bool = ...,
        **kwargs: Any
    ) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Number(Schema):
    integer_only: bool = ...
    errors: List[Error] = ...
    minimum: int = ...
    maximum: int = ...
    exclusive_minimum: int = ...
    exclusive_maximum: int = ...
    multiple_of: Optional[int] = ...
    def __init__(
        self,
        minimum: Optional[int] = ...,
        maximum: Optional[int] = ...,
        exclusive_minimum: bool = ...,
        exclusive_maximum: bool = ...,
        multiple_of: Optional[int] = ...,
        **kwargs: Any
    ) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Integer(Number):
    errors: List[Error] = ...
    integer_only: bool = ...

class String(Schema):
    errors: List[Error] = ...
    max_length: Any = ...
    min_length: Any = ...
    pattern: str = ...
    format: str = ...
    pattern_regex: Pattern[str] = ...
    def __init__(
        self,
        max_length: Optional[int] = ...,
        min_length: Optional[int] = ...,
        pattern: Optional[str] = ...,
        format: Optional[Any] = ...,
        **kwargs: Any
    ) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Boolean(Schema):
    errors: List[Error] = ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Null(Schema):
    errors: List[Error] = ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Enum(Schema):
    errors: List[Error] = ...
    enum: Any = ...
    exact: Any = ...
    def __init__(self, enum: Any, **kwargs: Any) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Anything(Schema):
    errors: List[Error] = ...
    types: Any = ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Union(Schema):
    errors: List[Error] = ...
    children: List[Schema] = ...
    def __init__(self, children: List[Schema], **kwargs: Any) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Intersection(Schema):
    children: List[Schema] = ...
    def __init__(self, children: List[Schema], **kwargs: Any) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class ExclusiveUnion(Schema):
    errors: List[Error] = ...
    children: List[Schema] = ...
    def __init__(self, children: List[Schema], **kwargs: Any) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Not(Schema):
    errors: List[Error] = ...
    child: Any = ...
    def __init__(self, child: Any, **kwargs: Any) -> None: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class Ref(Schema):
    ref_name: Any = ...
    def __init__(self, ref_name: Any) -> None: ...
    def dereference(self, context: Any) -> Any: ...
    def validate(self, value: Any, context: Optional[Any] = ...) -> List[Error]: ...

class RefSpace(Schema):
    refs: Any = ...
    root: Any = ...
    root_validator: Any = ...
    def __init__(self, refs: Any, root: Any) -> None: ...
    def validate(self, value: Any) -> List[Error]: ...



from typing import Any, Optional

def manager_repr(value: Any) -> Any: ...
def smart_repr(value: Any) -> Any: ...
def field_repr(field: Any, force_many: bool = ...) -> Any: ...
def serializer_repr(serializer: Any, indent: Any, force_many: Optional[Any] = ...) -> Any: ...
def list_repr(serializer: Any, indent: Any) -> Any: ...



from typing import List, Optional, Pattern, Union

from django.urls.resolvers import RoutePattern, URLPattern, URLResolver

def apply_suffix_patterns(
    urlpatterns: List[Union[URLResolver, RoutePattern, URLPattern, Pattern[str]]],
    suffix_pattern: Union[str, Pattern[str]],
    suffix_required: bool,
    suffix_route: Optional[str] = ...,
) -> List[URLPattern]: ...
def format_suffix_patterns(
    urlpatterns: List[Union[URLResolver, RoutePattern, URLPattern, Pattern[str]]],
    suffix_required: bool = ...,
    allowed: Optional[List[Union[URLPattern, Pattern[str], str]]] = ...,
) -> List[URLPattern]: ...



from typing import Any

def link_sorting_key(link_item: Any) -> Any: ...
def get_links_from_document(node: Any, keys: Any = ...) -> Any: ...
def get_method(link: Any) -> Any: ...
def get_encoding(link: Any) -> Any: ...
def get_location(link: Any, field: Any) -> Any: ...



from coreschema.encodings.html import render_to_form as render_to_form
from coreschema.schemas import Anything as Anything
from coreschema.schemas import Array as Array
from coreschema.schemas import Boolean as Boolean
from coreschema.schemas import Enum as Enum
from coreschema.schemas import ExclusiveUnion as ExclusiveUnion
from coreschema.schemas import Integer as Integer
from coreschema.schemas import Intersection as Intersection
from coreschema.schemas import Not as Not
from coreschema.schemas import Null as Null
from coreschema.schemas import Number as Number
from coreschema.schemas import Object as Object
from coreschema.schemas import Ref as Ref
from coreschema.schemas import RefSpace as RefSpace
from coreschema.schemas import String as String
from coreschema.schemas import Union as Union



from typing import Any

from openapi_codec.utils import get_encoding as get_encoding
from openapi_codec.utils import get_links_from_document as get_links_from_document
from openapi_codec.utils import get_location as get_location
from openapi_codec.utils import get_method as get_method

def generate_swagger_object(document: Any) -> Any: ...



from typing import Any, Iterable, List, Optional, Pattern, Sequence, Tuple, Type, Union

from django.db.models.base import Model

import coreapi
from rest_framework.request import Request
from rest_framework.views import APIView

def common_path(paths: Iterable[str]) -> str: ...
def get_pk_name(model: Type[Model]) -> str: ...
def is_api_view(callback: Any) -> bool: ...

_APIEndpoint = Tuple[str, str, Any]

class EndpointEnumerator:
    patterns: List[Any]
    def __init__(self, patterns: Optional[List[Any]] = ..., urlconf: Optional[str] = ...) -> None: ...
    def get_api_endpoints(self, patterns: Optional[List[Any]] = ..., prefix: str = ...) -> List[_APIEndpoint]: ...
    def get_path_from_regex(self, path_regex: Union[Pattern[str], str]) -> str: ...
    def should_include_endpoint(self, path: str, callback: Any) -> bool: ...
    def get_allowed_methods(self, callback: Any) -> List[str]: ...

class BaseSchemaGenerator:
    endpoint_inspector_cls: Type[EndpointEnumerator] = ...
    coerce_path_pk: Optional[bool] = ...
    patterns: List[Any] = ...
    urlconf: Optional[str] = ...
    title: Optional[str] = ...
    description: Optional[str] = ...
    version: Optional[str] = ...
    url: Optional[str] = ...
    endpoints: Optional[Sequence[_APIEndpoint]] = ...
    def __init__(
        self,
        title: Optional[str] = ...,
        url: Optional[str] = ...,
        description: Optional[str] = ...,
        patterns: Optional[List[Any]] = ...,
        urlconf: Optional[str] = ...,
        version: Optional[str] = ...,
    ) -> None: ...
    def create_view(self, callback: Any, method: str, request: Optional[Request] = ...) -> Any: ...
    def coerce_path(self, path: str, method: str, view: APIView) -> str: ...
    def get_schema(self, request: Optional[Request] = ..., public: bool = ...) -> Optional[coreapi.Document]: ...
    def has_view_permissions(self, path: str, method: str, view: APIView) -> bool: ...



from typing import Any, Dict, Type

from rest_framework import serializers
from rest_framework.request import Request
from rest_framework.serializers import BaseSerializer
from rest_framework.utils.field_mapping import ClassLookupDict
from rest_framework.views import APIView

class BaseMetadata:
    def determine_metadata(self, request: Request, view: APIView) -> Dict[str, Any]: ...

class SimpleMetadata(BaseMetadata):
    label_lookup: ClassLookupDict[Type[serializers.Field[Any, Any, Any, Any]], str]
    def determine_actions(self, request: Request, view: APIView) -> Dict[str, Any]: ...
    def get_serializer_info(self, serializer: BaseSerializer) -> Dict[str, Dict[str, Any]]: ...
    def get_field_info(self, field: serializers.Field[Any, Any, Any, Any]) -> Dict[str, Any]: ...



from typing import Any, Dict, NamedTuple, Optional, Type

from django.db.models import Model
from django.db.models.fields import Field
from django.db.models.fields.related import RelatedField

class RelationInfo(NamedTuple):
    model_field: Optional[RelatedField[Any, Any]]
    related_model: Type[Model]
    to_many: bool
    to_field: str
    has_through_model: bool
    reverse: bool

class FieldInfo(NamedTuple):
    pk: Field[Any, Any]
    fields: Dict[str, Field[Any, Any]]
    forward_relations: Dict[str, RelationInfo]
    reverse_relations: Dict[str, RelationInfo]
    fields_and_pk: Dict[str, Field[Any, Any]]
    relations: Dict[str, RelationInfo]

def get_field_info(model: Type[Model]) -> FieldInfo: ...
def is_abstract_model(model: Type[Model]) -> bool: ...



from typing import Any

def datetime_formats(formats: Any) -> Any: ...
def date_formats(formats: Any) -> Any: ...
def time_formats(formats: Any) -> Any: ...
def humanize_strptime(format_string: Any) -> Any: ...



from typing import Any, Container, Iterable, MutableMapping, NoReturn, Optional, TypeVar

from django.db.models import Model, QuerySet

from rest_framework.fields import Field
from rest_framework.serializers import BaseSerializer

_T = TypeVar("_T", bound=Model)

def qs_exists(queryset: QuerySet[Any]) -> bool: ...
def qs_filter(queryset: QuerySet[_T], **kwargs: Any) -> QuerySet[_T]: ...

class UniqueValidator:
    message: str = ...
    requires_context: bool = ...
    queryset: QuerySet[Any]
    lookup: str
    def __init__(self, queryset: QuerySet[Any], message: Optional[str] = ..., lookup: str = ...) -> None: ...
    def filter_queryset(self, value: Any, queryset: QuerySet[_T], field_name: str) -> QuerySet[_T]: ...
    def exclude_current_instance(self, queryset: QuerySet[_T], instance: _T) -> QuerySet[_T]: ...
    def __call__(self, value: Any, serializer_field: Field[Any, Any, Any, Any]) -> NoReturn: ...

class UniqueTogetherValidator:
    message: str = ...
    missing_message: str = ...
    requires_context: bool = ...
    queryset: QuerySet[Any]
    fields: Iterable[str]
    def __init__(self, queryset: QuerySet[Any], fields: Iterable[str], message: Optional[str] = ...) -> None: ...
    def enforce_required_fields(self, attrs: Container[str], serializer: BaseSerializer) -> NoReturn: ...
    def filter_queryset(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], serializer: BaseSerializer
    ) -> QuerySet[_T]: ...
    def exclude_current_instance(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], instance: _T
    ) -> QuerySet[_T]: ...
    def __call__(self, attrs: MutableMapping[str, Any], serializer: BaseSerializer) -> NoReturn: ...

class ProhibitSurrogateCharactersValidator:
    message: str = ...
    code: str = ...
    def __call__(self, value: Any) -> NoReturn: ...

class BaseUniqueForValidator:
    message: str = ...
    missing_message: str = ...
    requires_context: bool = ...
    queryset: QuerySet[Any]
    field: str
    date_field: str
    def __init__(self, queryset: QuerySet[Any], field: str, date_field: str, message: Optional[str] = ...) -> None: ...
    def enforce_required_fields(self, attrs: Container[str]) -> NoReturn: ...
    def filter_queryset(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], field_name: str, date_field_name: str
    ) -> QuerySet[_T]: ...
    def exclude_current_instance(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], instance: Model
    ) -> QuerySet[_T]: ...
    def __call__(self, attrs: MutableMapping[str, Any], serializer: BaseSerializer) -> NoReturn: ...

class UniqueForDateValidator(BaseUniqueForValidator): ...
class UniqueForMonthValidator(BaseUniqueForValidator): ...
class UniqueForYearValidator(BaseUniqueForValidator): ...



from typing import Any, Dict, Generic, MutableMapping, Sequence, Type, TypeVar

from django.db import models

NUMERIC_FIELD_TYPES: Sequence[Type[models.Field[Any, Any]]] = ...

_K = TypeVar("_K", bound=type)
_V = TypeVar("_V")

class ClassLookupDict(Generic[_K, _V]):
    mapping: MutableMapping[Type[_K], _V]
    def __init__(self, mapping: MutableMapping[Type[_K], _V]): ...
    def __getitem__(self, key: _K) -> _V: ...
    def __setitem__(self, key: _K, value: _V) -> None: ...

def needs_label(model_field: models.Field[Any, Any], field_name: str) -> bool: ...
def get_detail_view_name(model: models.Model) -> str: ...
def get_field_kwargs(field_name: str, model_field: models.Field[Any, Any]) -> Dict[str, Any]: ...
def get_relation_kwargs(field_name: str, relation_info: Any) -> Dict[str, Any]: ...
def get_nested_relation_kwargs(relation_info: Any) -> Dict[str, Any]: ...
def get_url_kwargs(model_field: models.Model) -> Dict[str, Any]: ...



from typing import Any, Callable

from rest_framework.authtoken.serializers import AuthTokenSerializer
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.views import APIView

class ObtainAuthToken(APIView):
    serializer_class = AuthTokenSerializer
    def post(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

obtain_auth_token: Callable[..., Any] = ...



from typing import Any, Optional

from openapi_codec import OpenAPICodec as _OpenAPICodec
from rest_framework.renderers import BaseRenderer

class OpenAPICodec(_OpenAPICodec):
    def encode(self, document: Any, **options: Any) -> Any: ...

class OpenAPIRenderer(BaseRenderer):
    media_type: str = ...
    charset: Any = ...
    format: str = ...
    def render(
        self, data: Any, accepted_media_type: Optional[Any] = ..., renderer_context: Optional[Any] = ...
    ) -> Any: ...
    def get_customizations(self) -> Any: ...

class SwaggerUIRenderer(BaseRenderer):
    media_type: str = ...
    format: str = ...
    template: str = ...
    charset: str = ...
    def render(
        self, data: Any, accepted_media_type: Optional[Any] = ..., renderer_context: Optional[Any] = ...
    ) -> Any: ...
    def set_context(self, data: Any, renderer_context: Any) -> None: ...
    def get_auth_urls(self) -> Any: ...
    def get_ui_settings(self) -> Any: ...



from typing import Any, Dict

from django.http.response import HttpResponse

from rest_framework.serializers import BaseSerializer

class CreateModelMixin:
    def create(self, request: Any, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def perform_create(self, serializer: BaseSerializer) -> None: ...
    def get_success_headers(self, data: Any) -> Dict[str, str]: ...

class ListModelMixin:
    def list(self, request: Any, *args: Any, **kwargs: Any) -> HttpResponse: ...

class RetrieveModelMixin:
    def retrieve(self, request: Any, *args: Any, **kwargs: Any) -> HttpResponse: ...

class UpdateModelMixin:
    def update(self, request: Any, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def perform_update(self, serializer: BaseSerializer) -> None: ...
    def partial_update(self, request: Any, *args: Any, **kwargs: Any) -> HttpResponse: ...

class DestroyModelMixin:
    def destroy(self, request: Any, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def perform_destroy(self, instance: Any) -> None: ...



from typing import Tuple, Type, Union

def force_bytes(string: Union[str, bytes]) -> bytes: ...
def force_text(string: Union[str, bytes]) -> str: ...

text_types: Tuple[Type[str]]
numeric_types: Tuple[Type[float], Type[int]]



from collections import OrderedDict
from typing import Any, Dict, Iterable, List, MutableMapping, Tuple, Union

from rest_framework.exceptions import ErrorDetail
from rest_framework.fields import Field
from rest_framework.serializers import BaseSerializer

class ReturnDict(OrderedDict[Any, Any]):
    serializer: BaseSerializer
    def __init__(self, serializer: BaseSerializer = ..., *args: Any, **kwargs: Any): ...
    def copy(self) -> ReturnDict: ...
    def __reduce__(self) -> Tuple[Dict[Any, Any], Tuple[Dict[Any, Any]]]: ...

class ReturnList(List[Any]):
    serializer: BaseSerializer
    def __init__(self, serializer: BaseSerializer = ..., *args: Any, **kwargs: Any): ...
    def __reduce__(self) -> Tuple[Dict[Any, Any], Tuple[Dict[Any, Any]]]: ...

class BoundField:
    """
    A field object that also includes `.value` and `.error` properties.
    Returned when iterating over a serializer instance,
    providing an API similar to Django forms and form fields.
    """

    value: Any
    fields: Dict[str, Field[Any, Any, Any, Any]]
    errors: List[ErrorDetail]
    def __init__(self, field: Field[Any, Any, Any, Any], value: Any, errors: List[ErrorDetail], prefix: str = ...): ...
    def __getattr__(self, attr_name: str) -> Any: ...
    def as_form_field(self) -> BoundField: ...

class JSONBoundField(BoundField): ...

class NestedBoundField(BoundField):
    def __iter__(self) -> Iterable[str]: ...
    def __getitem__(self, key: str) -> Union[BoundField, NestedBoundField]: ...

class BindingDict(MutableMapping[str, Field[Any, Any, Any, Any]]):
    serializer: BaseSerializer
    fields: OrderedDict[str, Field[Any, Any, Any, Any]]
    def __init__(self, serializer: BaseSerializer): ...
    def __setitem__(self, key: str, field: Field[Any, Any, Any, Any]) -> None: ...
    def __getitem__(self, key: str) -> Field[Any, Any, Any, Any]: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterable[Field[Any, Any, Any, Any]]: ...  # type: ignore[override]
    def __len__(self) -> int: ...



from typing import Any, Dict, List, Protocol, Sequence, Type, Union

from django.db.models import Model, QuerySet

from rest_framework.views import APIView

SAFE_METHODS: Sequence[str]

class _SupportsHasPermission(Protocol):
    def has_permission(self, request: Any, view: APIView) -> bool: ...
    def has_object_permission(self, request: Any, view: APIView, obj: Any) -> bool: ...

_PermissionClass = Union[Type[BasePermission], OperandHolder, SingleOperandHolder]

class OperationHolderMixin:
    def __and__(self, other: _PermissionClass) -> OperandHolder: ...
    def __or__(self, other: _PermissionClass) -> OperandHolder: ...
    def __rand__(self, other: _PermissionClass) -> OperandHolder: ...
    def __ror__(self, other: _PermissionClass) -> OperandHolder: ...
    def __invert__(self) -> SingleOperandHolder: ...

class SingleOperandHolder(OperationHolderMixin):
    operator_class: _SupportsHasPermission
    op1_class: _PermissionClass
    def __init__(self, operator_class: _SupportsHasPermission, op1_class: _PermissionClass): ...
    def __call__(self, *args: Any, **kwargs: Any) -> _SupportsHasPermission: ...

class OperandHolder(OperationHolderMixin):
    operator_class: _SupportsHasPermission
    op1_class: _PermissionClass
    op2_class: _PermissionClass
    def __init__(
        self, operator_class: _SupportsHasPermission, op1_class: _PermissionClass, op2_class: _PermissionClass
    ): ...
    def __call__(self, *args: Any, **kwargs: Any) -> _SupportsHasPermission: ...

class AND(_SupportsHasPermission):
    def __init__(self, op1: _SupportsHasPermission, op2: _SupportsHasPermission) -> None: ...

class OR(_SupportsHasPermission):
    def __init__(self, op1: _SupportsHasPermission, op2: _SupportsHasPermission) -> None: ...

class NOT(_SupportsHasPermission):
    def __init__(self, op1: _SupportsHasPermission) -> None: ...

class BasePermissionMetaclass(OperationHolderMixin, type): ...
class BasePermission(_SupportsHasPermission, metaclass=BasePermissionMetaclass): ...
class AllowAny(BasePermission): ...
class IsAuthenticated(BasePermission): ...
class IsAdminUser(BasePermission): ...
class IsAuthenticatedOrReadOnly(BasePermission): ...

class DjangoModelPermissions(BasePermission):
    perms_map: Dict[str, List[str]] = ...
    authenticated_users_only: bool = ...
    def get_required_permissions(self, method: str, model_cls: Type[Model]) -> List[str]: ...
    def _queryset(self, view: APIView) -> QuerySet[Any]: ...

class DjangoModelPermissionsOrAnonReadOnly(DjangoModelPermissions): ...

class DjangoObjectPermissions(DjangoModelPermissions):
    def get_required_object_permissions(self, method: str, model_cls: Type[Model]) -> List[str]: ...



from typing import Any

from coreschema import Array as Array
from coreschema import Boolean as Boolean
from coreschema import Enum as Enum
from coreschema import Integer as Integer
from coreschema import Number as Number
from coreschema import Object as Object
from coreschema import String as String

env: Any

def render_to_form(schema: Any) -> Any: ...
def determine_html_template(schema: Any) -> Any: ...
def get_textarea_value(schema: Any) -> Any: ...
def get_attrs(schema: Any) -> Any: ...



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['coreapi-stubs',
 'coreschema-stubs',
 'openapi_codec-stubs',
 'rest_framework-stubs',
 'rest_framework_swagger-stubs']

package_data = \
{'': ['*'],
 'coreapi-stubs': ['codecs/*'],
 'coreschema-stubs': ['encodings/*'],
 'rest_framework-stubs': ['authtoken/*',
                          'authtoken/management/*',
                          'authtoken/management/commands/*',
                          'management/commands/*',
                          'schemas/*',
                          'templatetags/*',
                          'utils/*']}

setup_kwargs = {
    'name': 'djangorestframework-types',
    'version': '0.4.0',
    'description': 'Type stubs for Django Rest Framework',
    'long_description': "# djangorestframework-types [![PyPI](https://img.shields.io/pypi/v/djangorestframework-types.svg)](https://pypi.org/project/djangorestframework-types/)\n\nType stubs for [Django Rest Framework](https://www.django-rest-framework.org).\n\n> Note: this project was forked from\n> <https://github.com/typeddjango/djangorestframework-stubs> with the goal of\n> removing the [`mypy`](https://github.com/python/mypy) plugin dependency so\n> that `mypy` can't [crash due to Django\n> config](https://github.com/typeddjango/django-stubs/issues/318), and that\n> non-`mypy` type checkers like\n> [`pyright`](https://github.com/microsoft/pyright) will work better with\n> Django Rest Framework.\n\n## install\n\n```bash\npip install djangorestframework-types\n```\n\n## related\n\n- <https://github.com/sbdchd/django-types>\n",
    'author': 'Steve Dignam',
    'author_email': 'steve@dignam.xyz',
    'url': 'https://github.com/sbdchd/djangorestframework-types',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Callable, Dict, Mapping, Optional, Sequence, Union

from typing_extensions import TypedDict

class DefaultsSettings(TypedDict, total=False):
    DEFAULT_RENDERER_CLASSES: Sequence[str]
    DEFAULT_PARSER_CLASSES: Sequence[str]
    DEFAULT_AUTHENTICATION_CLASSES: Sequence[str]
    DEFAULT_PERMISSION_CLASSES: Sequence[str]
    DEFAULT_THROTTLE_CLASSES: Sequence[str]
    DEFAULT_CONTENT_NEGOTIATION_CLASS: str
    DEFAULT_METADATA_CLASS: str
    DEFAULT_VERSIONING_CLASS: Optional[str]
    DEFAULT_PAGINATION_CLASS: Optional[str]
    DEFAULT_FILTER_BACKENDS: Sequence[str]
    DEFAULT_SCHEMA_CLASS: str
    DEFAULT_THROTTLE_RATES: Dict[str, Optional[Union[float, int]]]
    NUM_PROXIES: Optional[int]
    PAGE_SIZE: Optional[int]
    SEARCH_PARAM: str
    ORDERING_PARAM: str
    DEFAULT_VERSION: Optional[str]
    ALLOWED_VERSIONS: Optional[str]
    VERSION_PARAM: str
    UNAUTHENTICATED_USER: str
    UNAUTHENTICATED_TOKEN: Optional[str]
    VIEW_NAME_FUNCTION: str
    VIEW_DESCRIPTION_FUNCTION: str
    EXCEPTION_HANDLER: Union[str, Callable[[Any, Any], Any]]
    NON_FIELD_ERRORS_KEY: str
    TEST_REQUEST_RENDERER_CLASSES: Sequence[str]
    TEST_REQUEST_DEFAULT_FORMAT: str
    URL_FORMAT_OVERRIDE: str
    FORMAT_SUFFIX_KWARG: str
    URL_FIELD_NAME: str
    DATE_FORMAT: str
    DATE_INPUT_FORMATS: Sequence[str]
    DATETIME_FORMAT: str
    DATETIME_INPUT_FORMATS: Sequence[str]
    TIME_FORMAT: str
    TIME_INPUT_FORMATS: Sequence[str]
    UNICODE_JSON: bool
    COMPACT_JSON: bool
    STRICT_JSON: bool
    COERCE_DECIMAL_TO_STRING: bool
    UPLOADED_FILES_USE_URL: bool
    HTML_SELECT_CUTOFF: int
    HTML_SELECT_CUTOFF_TEXT: str
    SCHEMA_COERCE_PATH_PK: bool
    SCHEMA_COERCE_METHOD_NAMES: Dict[str, str]

DEFAULTS: DefaultsSettings = ...
IMPORT_STRINGS: Sequence[str] = ...
REMOVED_SETTINGS: Sequence[str] = ...

def perform_import(val: Optional[Any], setting_name: str) -> Optional[Any]: ...
def import_from_string(val: Optional[Any], setting_name: str) -> Any: ...

class APISettings:
    defaults: DefaultsSettings
    import_strings: Sequence[str]
    def __init__(
        self,
        user_settings: Optional[DefaultsSettings] = ...,
        defaults: Optional[DefaultsSettings] = ...,
        import_strings: Optional[Sequence[str]] = ...,
    ): ...
    @property
    def user_settings(self) -> Mapping[str, Any]: ...
    def __getattr__(self, attr: str) -> Any: ...
    def __check_user_settings(self, user_settings: Mapping[str, Any]) -> Mapping[str, Any]: ...
    def reload(self) -> None: ...

class _Settings(APISettings):
    DEFAULT_RENDERER_CLASSES: Sequence[str] = ...
    DEFAULT_PARSER_CLASSES: Sequence[str] = ...
    DEFAULT_AUTHENTICATION_CLASSES: Sequence[str] = ...
    DEFAULT_PERMISSION_CLASSES: Sequence[str] = ...
    DEFAULT_THROTTLE_CLASSES: Sequence[str] = ...
    DEFAULT_CONTENT_NEGOTIATION_CLASS: str = ...
    DEFAULT_METADATA_CLASS: str = ...
    DEFAULT_VERSIONING_CLASS: Optional[str] = ...
    DEFAULT_PAGINATION_CLASS: Optional[str] = ...
    DEFAULT_FILTER_BACKENDS: Sequence[str] = ...
    DEFAULT_SCHEMA_CLASS: str = ...
    DEFAULT_THROTTLE_RATES: Dict[str, Optional[Union[float, int]]] = ...
    NUM_PROXIES: Optional[int] = ...
    PAGE_SIZE: Optional[int] = ...
    SEARCH_PARAM: str = ...
    ORDERING_PARAM: str = ...
    DEFAULT_VERSION: Optional[str] = ...
    ALLOWED_VERSIONS: Optional[str] = ...
    VERSION_PARAM: str = ...
    UNAUTHENTICATED_USER: str = ...
    UNAUTHENTICATED_TOKEN: Optional[str] = ...
    VIEW_NAME_FUNCTION: str = ...
    VIEW_DESCRIPTION_FUNCTION: str = ...
    EXCEPTION_HANDLER: Union[str, Callable[[Any, Any], Any]] = ...
    NON_FIELD_ERRORS_KEY: str = ...
    TEST_REQUEST_RENDERER_CLASSES: Sequence[str] = ...
    TEST_REQUEST_DEFAULT_FORMAT: str = ...
    URL_FORMAT_OVERRIDE: str = ...
    FORMAT_SUFFIX_KWARG: str = ...
    URL_FIELD_NAME: str = ...
    DATE_FORMAT: str = ...
    DATE_INPUT_FORMATS: Sequence[str] = ...
    DATETIME_FORMAT: str = ...
    DATETIME_INPUT_FORMATS: Sequence[str] = ...
    TIME_FORMAT: str = ...
    TIME_INPUT_FORMATS: Sequence[str] = ...
    UNICODE_JSON: bool = ...
    COMPACT_JSON: bool = ...
    STRICT_JSON: bool = ...
    COERCE_DECIMAL_TO_STRING: bool = ...
    UPLOADED_FILES_USE_URL: bool = ...
    HTML_SELECT_CUTOFF: int = ...
    HTML_SELECT_CUTOFF_TEXT: str = ...
    SCHEMA_COERCE_PATH_PK: bool = ...
    SCHEMA_COERCE_METHOD_NAMES: Dict[str, str] = ...

api_settings: _Settings = ...

def reload_api_settings(*args: Any, **kwargs: Any) -> None: ...



from typing import Any, Optional, Tuple, Union

from django.db.models import QuerySet

def pygments_css(style: Any) -> Optional[str]: ...
def pygments_highlight(text: str, lang: str, style: Any) -> Any: ...
def md_filter_add_syntax_highlight(md: Any) -> bool: ...
def unicode_http_header(value: Union[str, bytes]) -> str: ...
def distinct(queryset: QuerySet[Any], base: Optional[QuerySet[Any]]) -> QuerySet[Any]: ...

SHORT_SEPARATORS: Tuple[str, str]
LONG_SEPARATORS: Tuple[str, str]
INDENT_SEPARATORS: Tuple[str, str]
